/***************************************************/
/*! \class FDNJot4LP
    \STK order 4 single input, single output feedback
    \delay network, with each delay line cascaded by 
    \a one-pole lowpass filter. There is also a tone
    \correction filter.

    This private class implements an order 4 SISO FDN
    with B = [1,1,1,1]^T, C = [1,1,1,1]^T, and feedback
    matrix A = A_4, where A_N = gA*(I_N - (2/N)*u_N*(u_N)^T).
    (I_N denotes the N by N identity matrix, and
    u_N = [1,...,1]^T, an N-vector of ones.) Each delay is
    cascaded by a lowpass filter according to the
    difference equation:
         y_i(n) = g_i(1-a_i)*x_i(n) + a_i*y_i(n-1)
    where the subscript i = {1,2,3,4} denotes each of the
    4 delay lines. A tone correction filter is applied
    to the FDN output and summed with the input sample
    to produce the final system output.

    You set the reverberation time at DC and at half the
    sampling rate, the gain coefficient gA of the A matrix,
    and the delay line lengths of each of the 4 delay lines
    during instantiation.

    Music 421, Spring Quarter 2003-2004
*/
/***************************************************/

#include "FDNJot4LP.h"
#include "Stk.h"
#include <stdio.h>
#include <math.h> // for basic math functions

FDNJot4LP :: FDNJot4LP(MY_FLOAT RTatDC, MY_FLOAT RTatPI, MY_FLOAT gainAVal, long *delayL)
{
  int i;
  long M[4]; // length of delay lines
  for (i=0;i<4;i++) {
    M[i] = delayL[i];
    delayline[i] = new Delay(M[i],M[i]);
  }

  // some variables need for calculating lowpass filter coefficients
  MY_FLOAT T, alpha;  
  T = 1/(Stk::sampleRate());
  if (RTatDC == 0.0) {
    alpha = 1.0; // to prevent dividing by 0
  } else {
    alpha = RTatPI/RTatDC;
  }

  /************************************/
  /* WRITE CODE HERE (set up filters) */
  /************************************/

  gA = gainAVal;
  this->clear();
}

FDNJot4LP :: ~FDNJot4LP()
{
  int i;
  for (i=0;i<4;i++) {
    delete delayline[i];
    delete lpFilter[i];
  }
  delete tcFilter;
}

void FDNJot4LP :: clear(void)
{
  int i,j;
  for (i=0;i<4;i++) {
    delayline[i]->clear();
    lpFilter[i]->clear();
  }
  tcFilter->clear();
  for (j=0;j<4;j++) {
    dl_inputs[j] = 0.0;
    dl_outputs[j] = 0.0;
    lp_outputs[j] = 0.0;
  }
  sum_lp_outputs = 0.0;
  tc_output = 0.0;
}

MY_FLOAT FDNJot4LP :: tick(MY_FLOAT sample)
{

  /********************/
  /* WRITE CODE HERE */
  /*******************/

}
